@@ -270,12 +270,6 @@ class Semaphore(SyncMethodMixin):
     acquired. If this happens, a warning is emitted but the internal state is
     not modified.
 
-    .. warning::
-
-        This implementation is still in an experimental state and subtle
-        changes in behavior may occur without any change in the major version
-        of this library.
-
     .. warning::
 
         This implementation is susceptible to lease overbooking in case of
